# from .kms_util import KMSUtil
# from .oss_util import OSSUtil
# from .s3_util import S3Util
#
# import str_util
# import properties_util
# import tar_util
#
# __all__ = [
#     "KMSUtil",
#     "OSSUtil",
#     "S3Util",
#     "str_util",
#     "properties_util",
#     "tar_util",
# ]


# -*- coding: utf-8 -*-
import logging
import os
import sys


from alibabacloud_mse20190531.client import Client as mse20190531Client
from alibabacloud_credentials.client import Client as CredentialClient
from alibabacloud_credentials.models import Config as CredentialConfig
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_mse20190531 import models as mse_20190531_models
from alibabacloud_tea_util import models as util_models
from .kms_util import KMSUtil


class NacosUtil:
    MSE_ENDPOINT = f'mse.cn-shanghai.aliyuncs.com'
    INSTANCE_ID = f'mse_prepaid_public_cn-7pp2io51d09'
    NAMESPACE_ID = None
    mse_client = None
    kms_util: KMSUtil = None

    def __init__(self, **kwargs):
        """
        :param kwargs:
        : server_addresses
        : namespace
        : access_key_id
        : access_key_secret
        : ram_role_name
        : aes_key_arn
        : encryption_context
        : kms_util
        """
        mse_endpoint = kwargs.get('mse_endpoint')
        instance_id = kwargs.get('instance_id')
        namespace_id = kwargs.get('namespace_id')
        access_key_id = kwargs.get('access_key_id')
        access_key_secret = kwargs.get('access_key_secret')
        ram_role_name = kwargs.get('ram_role_name')
        encryption_context = kwargs.get('encryption_context')
        kms_endpoint = kwargs.get('kms_endpoint')

        if not (access_key_id and access_key_secret) and not ram_role_name:
            raise Exception("RamRoleName or AK/SK cannot be null")
        if mse_endpoint and len(mse_endpoint) > 0:
            self.MSE_ENDPOINT = mse_endpoint
        if instance_id and len(instance_id) > 0:
            self.INSTANCE_ID = instance_id
        if namespace_id and len(namespace_id) > 0:
            self.NAMESPACE_ID = namespace_id
        if ram_role_name and len(ram_role_name) > 0:
            credential = CredentialClient(CredentialConfig(
                type='ecs_ram_role',
                role_name=ram_role_name
            ))
        else:
            credential = CredentialClient(CredentialConfig(
                type='access_key',
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
            ))
        config = open_api_models.Config(
            credential=credential, endpoint=self.MSE_ENDPOINT
        )
        self.mse_client = mse20190531Client(config)
        self.kms_util = KMSUtil(kms_endpoint, access_key_id, access_key_secret, ram_role_name, encryption_context=encryption_context)

    def get_config(self, data_id=None, group='DEFAULT_GROUP'):
        if data_id and len(data_id) > 0:
            self.data_id = data_id
        if self.data_id and len(self.data_id) > 0:
            get_nacos_config_request = mse_20190531_models.GetNacosConfigRequest(
                instance_id=self.INSTANCE_ID,
                namespace_id=self.NAMESPACE_ID,
                group=group,
                data_id=data_id
            )
            runtime = util_models.RuntimeOptions()
            response = self.mse_client.get_nacos_config_with_options(get_nacos_config_request, runtime)
            context = response.body.configuration.content
            if context:
                return self.kms_util.decrypt_envelope(context, data_id.split(".")[-1])


        #
        #     if self.data_id.endswith('.properties'):
        #         return json.dumps(props_util.properties_to_dict(nacos_props))
        #     return json.dumps(nacos_props)
        # raise Exception("data_id cannot be null without a default value")

#
# nacos_util = NacosUtil(namespace_id="0238e844-3fef-436a-a683-a2eecb888e29",
#                        access_key_id=os.environ.get('ALIBABA_CLOUD_ACCESS_KEY_ID'),
#                        access_key_secret=os.environ.get('ALIBABA_CLOUD_ACCESS_KEY_SECRET'),
#                        ram_role_name=os.environ.get('ALIBABA_CLOUD_RAM_ROLE_NAME')
#                                                          )
# print(props_util.flatten_dict(nacos_util.get_config('cipher-ds-dataworks-api-prod.yaml')))
# print(props_util.get_properties_by_prefix(nacos_util.get_config('cipher-kms-aes-256-ds-toolbox.properties'), 'nebula.ds_rec.'))


import logging
import os
import oss2

from .str_util import extract_bucket, extract_object_key


# def extract_path(filepath):
#     """
#     D:/mnt/data/private/model/20230607 => /mnt/data/private/model/20230607
#     oss://opg211-dev-ds-export/private/model/20230607 => /mnt/data/private/model/20230607
#     :param filepath:
#     :return:
#     """


class OSSUtil:
    ACCESS_KEY_ID = None
    ACCESS_KEY_SECRET = None
    ENDPOINT = None

    def __init__(self, access_key_id, access_key_secret, endpoint):
        self.ACCESS_KEY_ID = access_key_id
        self.ACCESS_KEY_SECRET = access_key_secret
        self.ENDPOINT = endpoint

    def get_object(self, remote_file):
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        try:
            if bucket_name:
                bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
                return str(bucket.get_object(object_key).read(), encoding='utf-8')
            else:
                logging.error("file: %s, format error, must be complete!", object_key)
        except Exception as e:
            logging.error(e)

    def put_object(self, object, remote_file):
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        if bucket_name:
            bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
            bucket.put_object(object_key, object)
            logging.info("file: %s upload completed!", object_key)
            return True
        else:
            logging.error("file: %s, format error, must be complete!", object_key)

    def upload_file(self, local_file, remote_file):
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        if bucket_name:
            bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
            with open(local_file, "rb") as f:
                bucket.put_object(object_key, f)
                logging.info("file: %s upload completed!", object_key)
                return True
        else:
            logging.error("file: %s, format error, must be complete!", object_key)

    def upload_dir(self, local_dir, remote_dir, excludes=[]):
        bucket_name = extract_bucket(remote_dir)
        object_path = extract_object_key(remote_dir)
        bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
        upload_files_num = 0
        if os.path.isdir(local_dir):
            for root, dir, files in os.walk(local_dir):
                for filename in files:
                    fullpath = os.path.join(root, filename)
                    object_key = object_path + (fullpath[len(local_dir):].replace('\\', '/'))
                    if not any(item and len(item) > 0 and item in fullpath for item in excludes):
                        with open(fullpath, "rb") as f:
                            bucket.put_object(object_key, f)
                            upload_files_num += 1
                            logging.info("file: %s upload completed!", object_key)
        logging.info("dir: %s upload completed! upload files num: %s", local_dir, upload_files_num)

    def download_file(self, remote_file, local_file, force=False):
        if not force and os.path.exists(local_file):
            logging.info("file: %s local already exists", local_file)
            return True
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
        object_stream = bucket.get_object(object_key)
        local_dir = os.path.dirname(local_file)
        if not os.path.exists(local_dir):
            os.makedirs(local_dir)
        with open(local_file, "wb") as f:
            f.write(object_stream.read())
            logging.info("file: %s download completed!", local_file)
            return True

    def download_dir(self, remote_dir, local_dir, ext='', excludes=[]):
        """
        :param remote_dir:
        :param local_dir:
        :param ext:
        :param excludes:
        :return:
        """
        bucket_name = extract_bucket(remote_dir)
        prefix = extract_object_key(remote_dir)
        bucket = oss2.Bucket(oss2.Auth(self.ACCESS_KEY_ID, self.ACCESS_KEY_SECRET), self.ENDPOINT, bucket_name)
        for obj in oss2.ObjectIterator(bucket, prefix=prefix):
            if not obj.key.endswith("/") and obj.key.endswith(ext):
                if not any(item and len(item) > 0 and item in obj.key for item in excludes):
                    local_file = os.path.join(local_dir, obj.key[len(prefix):]).replace('\\', '/')
                    local_path = os.path.dirname(local_file)
                    if not os.path.exists(local_path):
                        os.makedirs(local_path)
                    object_stream = bucket.get_object(obj.key)
                    with open(local_file, "wb") as f:
                        f.write(object_stream.read())
                        logging.info("file: %s download completed!", obj.key)
        logging.info("dir: %s download completed!", remote_dir)

# 调用示例
# uploaded = upload_file('D:/mnt/2023-05-06-epg.csv', 'temp/remote_file.csv')
# downloaded = download_file('temp/remote_file.csv', 'D:/mnt/local_file_001.csv')
# upload_dir('D:/mnt/data/private/temp', 'temp/test_dir', [''])
# print(os.path.dirname('D:/mnt/data/private/test/remote_file.csv'))
# print(extract_dir("oss://opg211-dev-ds-export/private/model/20230607"))
# print(extract_dir("s3://opg211-dev-ds-export/private/model/20230607"))



import time
from datetime import datetime


class LRUCache:
    def __init__(self, capacity: int, cache_ttl_ms=-1, cache_empty=False):
        self.capacity = capacity
        self.cache = {}
        self.lru = []
        self.init_ms = LRUCache.current_ms()
        self.cache_ttl_ms = cache_ttl_ms
        self.cache_empty = cache_empty

    @staticmethod
    def current_ms():
        return round(time.time() * 1000)

    def has(self, key):
        return key in self.cache

    def get(self, key):
        if key not in self.cache:
            return
        value = self.cache[key]
        if self.cache_ttl_ms < 0:
            self.lru.remove(key)
            self.lru.append(key)
            return value
        elif value and isinstance(value, list):
            if LRUCache.current_ms() - self.init_ms - int(value[1]) > self.cache_ttl_ms:
                del self.cache[key]
                self.lru.remove(key)
            else:
                self.lru.remove(key)
                self.lru.append(key)
                return value[0]

    def put(self, key, value) -> None:
        if self.cache_empty or value is not None:
            if key in self.cache:
                self.lru.remove(key)
            if len(self.cache) >= self.capacity:
                del self.cache[self.lru[0]]
                self.lru.pop(0)
            # If the ttl less than 0, store the value directly
            # Else store the value and different time from init
            self.cache[key] = value if self.cache_ttl_ms < 0 else [value, LRUCache.current_ms() - self.init_ms]
            self.lru.append(key)




def concat(s, n, sep=","):
    result = ''
    for i in range(n - 1):
        result = result + s + sep
    return result + s


def extract_bucket(path):
    if path and len(path) > 6:
        start = path.find('//') + 2
        end = path.find('/', 6)
        return path[start:end]


def extract_object_key(path):
    if path and len(path) > 6:
        start = path.find('/', 6) + 1
        return path[start:]


def extract_object_name(path):
    if path and len(path) > 6:
        return path.split('/')[-1]



# -*- coding: utf-8 -*-
import os
import sys
import logging
import re
import base64

from alibabacloud_kms20160120.client import Client as Kms20160120Client
from alibabacloud_credentials.client import Client as CredentialClient
from alibabacloud_credentials.client import Config as CredentialConfig
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_kms20160120 import models as kms_20160120_models
from alibabacloud_tea_util import models as util_models


class KMSUtil:
    ENCRYPTED_PROPERTY_PREFIX = "{cipher}"
    ENCRYPTED_PATTERN = "^(\\((?P<context>.*)\\)|\\[(?P<options>.*)]){0,2}(?P<cipher>.*)$"
    UTF_8 = "utf-8"
    ENDPOINT = 'kms.cn-shanghai.aliyuncs.com'
    ENCRYPTION_CONTEXT = {"ValidKey": "@@JL*%$DF@VS"}
    kms_client = None

    def __init__(self, endpoint=None, access_key_id=None, access_key_secret=None, ram_role_name=None, encryption_context=None):
        if not (access_key_id and access_key_secret) and not ram_role_name:
            raise Exception("RamRoleName or AK/SK cannot be null")
        if endpoint and len(endpoint) > 0:
            self.ENDPOINT = endpoint
        if encryption_context and len(encryption_context) > 0:
            self.ENCRYPTION_CONTEXT = encryption_context
        if ram_role_name and len(ram_role_name) > 0:
            credential = CredentialClient(CredentialConfig(
                type='ecs_ram_role',
                role_name=ram_role_name
            ))
        else:
            credential = CredentialClient(CredentialConfig(
                type='access_key',
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
            ))
        config = open_api_models.Config(
            credential=credential, endpoint=self.ENDPOINT
        )
        self.kms_client = Kms20160120Client(config)

    @staticmethod
    def parse_encrypted_token(cipher_text: str):
        temp = cipher_text.strip()
        if len(temp) > 0:
            if temp.startswith(KMSUtil.ENCRYPTED_PROPERTY_PREFIX):
                temp = temp[len(KMSUtil.ENCRYPTED_PROPERTY_PREFIX):]
            match_group = re.search(KMSUtil.ENCRYPTED_PATTERN, temp).groupdict()
            return KMSUtil.parse_encryption_context(match_group.get("context")), match_group.get("cipher")

    @staticmethod
    def parse_encryption_context(context: str):
        temp = context.strip()
        if temp and len(temp) > 0:
            dict = {}
            for pair in temp.split(","):
                kv = pair.split("=")
                if len(kv) > 1:
                    dict[kv[0]] = str(base64.b64decode(kv[1]), KMSUtil.UTF_8)
            return dict


    def encrypt(self, key_id, plain_text):
        encrypt_request = kms_20160120_models.EncryptRequest(
            key_id=key_id,
            plaintext=base64.b64encode(plain_text.encode('utf-8')).decode('utf-8'),
            encryption_context=self.ENCRYPTION_CONTEXT,
        )
        runtime = util_models.RuntimeOptions()
        response = self.kms_client.encrypt_with_options(encrypt_request, runtime)
        return response.body.ciphertext_blob


    def decrypt(self, cipher_text):
        decrypt_request = kms_20160120_models.DecryptRequest(
            encryption_context=self.ENCRYPTION_CONTEXT,
            ciphertext_blob=cipher_text
        )
        runtime = util_models.RuntimeOptions()
        response = self.kms_client.decrypt_with_options(decrypt_request, runtime)
        return base64.b64decode(response.body.plaintext).decode('utf-8')


    def decrypt_envelope(self, context: str, context_type: str):
        if context:
            if "properties" == context_type.lower().strip():
                from .properties_util import parse_properties
                cipher_props = parse_properties(context.split("\n"))
                plain_props = {}
                if cipher_props:
                    for key, value in cipher_props.items():
                        if str(value).startswith(self.ENCRYPTED_PROPERTY_PREFIX):
                            plain_props[key] = self.decrypt(str(value)[str(value).rfind(')') + 1:])
                        else:
                            plain_props[key] = value
                return plain_props
            elif "yaml" == context_type.lower().strip():
                import yaml
                cipher_yaml = yaml.load(context, Loader=yaml.FullLoader)
                KMSUtil.decrypt_dict(cipher_yaml, self.decrypt)
                return cipher_yaml

    @staticmethod
    def decrypt_dict(data, _func):
        if data and isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, dict):
                    KMSUtil.decrypt_dict(value, _func)
                elif isinstance(value, str) and str(value).startswith(KMSUtil.ENCRYPTED_PROPERTY_PREFIX):
                    context, cipher = KMSUtil.parse_encrypted_token(value)
                    data[key] = _func(cipher)
                else:
                    data[key] = value

#
# kms_util = KMSUtil(access_key_id=os.environ.get('ALIBABA_CLOUD_ACCESS_KEY_ID'),
#                    access_key_secret=os.environ.get('ALIBABA_CLOUD_ACCESS_KEY_SECRET'),
#                    ram_role_name=os.environ.get('ALIBABA_CLOUD_RAM_ROLE_NAME'))
# print(kms_util.encrypt("9900c848-5626-4f64-9eb3-ec91493852d3", "nebula"))


# encoding: utf-8
import pytz
import time
import datetime
import pandas as pd
import requests
import json
import re
import logging


if __name__ == '__main__':
    ram_role_name = 'ecs-opg211-prod-ds-rec'
    url = 'http://100.100.100.200/latest/meta-data/ram/security-credentials/%s'
    logging.info(url)
    headers = {
        "Content-Type": "application/json; charset=UTF-8"
    }

    res = requests.get(url % ram_role_name, headers=headers)
    if res.ok:
        result_json = json.dumps(res.text)
        print(result_json['AccessKeyId'], result_json['AccessKeySecret'])







import logging


def parse_jdbc_url(jdbc_url):
    from urllib.parse import urlparse, parse_qs

    """
    解析 MySQL JDBC URL 并返回 PyMySQL 连接所需的参数
    支持格式：
      1. jdbc:mysql://user:pass@host:port/database
      2. jdbc:mysql://host:port/database?user=xxx&password=xxx
    """
    if not (jdbc_url.startswith("jdbc:") or jdbc_url.startswith("mysql+")):
        raise ValueError("Invalid JDBC URL: Must start with 'jdbc:' or 'mysql+'")

    # 去掉 jdbc: 前缀并解析
    parsed = urlparse(jdbc_url[5:])  # 移除 "jdbc:"

    # 验证必要参数
    if not parsed.hostname:
        raise ValueError("Invalid JDBC URL: Host not found")

    # 提取基础参数
    host = parsed.hostname
    port = parsed.port or 3306  # 默认端口 3306
    database = parsed.path[1:] if parsed.path else None  # 移除开头的 '/'

    # 优先使用 URL 中的用户认证 (user:pass@host)
    user = parsed.username
    password = parsed.password

    # 次优使用查询参数 (?user=xx&password=xx)
    query_params = parse_qs(parsed.query)
    if user is None and "user" in query_params:
        user = query_params["user"][0]
    if password is None and "password" in query_params:
        password = query_params["password"][0]

    return {
        "host": host,
        "port": port,
        "database": database,
        "user": user,
        "password": password
    }


def pandas_upsert_to_postgres(engine, df, schema, table_name, conflict_columns):
    """
    使用 Pandas 在 PostgreSQL 中执行 Upsert 操作

    参数:
        df: 要插入/更新的 DataFrame
        table_name: 目标表名
        conflict_columns: 用于检测冲突的列名列表
        engine: SQLAlchemy 引擎对象
    """
    from sqlalchemy import text, inspect
    from sqlalchemy.exc import SQLAlchemyError
    try:
        # 创建临时表名
        temp_table = f"{table_name}_temp"
        inspector = inspect(engine)
        if not inspector.has_table(temp_table, schema=schema):
            with engine.connect() as conn:
                # 不生效
                # conn.execute(f"CALL hg_create_table_like('{schema}.{temp_table}', 'select * from {schema}.{table_name}');")
                conn.execute(f"create table {schema}.{temp_table} as select * from {schema}.{table_name} where false")
        with engine.connect() as conn:
            conn.execute(text(f"TRUNCATE TABLE {schema}.{temp_table}"))

        # 将数据写入临时表
        df.to_sql(
            name=temp_table,
            schema=schema,
            con=engine,
            if_exists='append',
            index=False
        )

        # 获取所有列名
        columns = ', '.join(df.columns)
        conflict_columns_str = ', '.join(conflict_columns)

        # 创建 SET 子句（排除冲突列）
        set_clause = ', '.join(
            [f"{col} = EXCLUDED.{col}"
             for col in df.columns
             if col not in conflict_columns]
        )

        # 构建 UPSERT 语句
        upsert_query = f"""
            INSERT INTO {schema}.{table_name} ({columns})
            SELECT {columns} FROM {schema}.{temp_table}
            ON CONFLICT ({conflict_columns_str}) 
            DO UPDATE SET {set_clause}
        """

        # 执行 UPSERT
        with engine.connect() as conn:
            conn.execute(text(upsert_query))

        logging.info("execute success upsert tablename=%s.%s, rows=%s", schema, table_name,  len(df))

    except SQLAlchemyError as e:
        logging.error(e)
        raise
    finally:
        logging.info("clear temp table: %s.%s", schema, temp_table)
        # 清理临时表
        try:
            with engine.connect() as conn:
                conn.execute(text(f"TRUNCATE TABLE {schema}.{temp_table}"))
        except Exception as e:
            logging.error("clear temp tablename=%s.%s error: %s", schema, temp_table, e)


# -*- coding: utf-8 -*-

import logging
import os
import boto3
from botocore.exceptions import NoCredentialsError
import itertools
from .str_util import extract_bucket, extract_object_key


class S3Util:
    # ACCESS_KEY_ID = None
    # ACCESS_KEY_SECRET = None
    # REGION = None
    client = None

    def __init__(self, access_key_id, access_key_secret, region):
        # self.ACCESS_KEY_ID = access_key_id
        # self.ACCESS_KEY_SECRET = access_key_secret
        # self.REGION = REGION
        self.client = boto3.client('s3', region_name=region, aws_access_key_id=access_key_id, aws_secret_access_key=access_key_secret)

    def upload_file(self, local_file, remote_file):
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        try:
            self.client.upload_file(local_file, bucket_name, object_key)
            logging.info("file: %s upload completed!", object_key)
            return True
        except FileNotFoundError:
            logging.error("file: %s local file not found", local_file)
            return False
        except NoCredentialsError:
            logging.error("AWS no Credential")
            return False

    def upload_dir(self, local_dir, remote_dir, excludes=[]):
        bucket_name = extract_bucket(remote_dir)
        object_path = extract_object_key(remote_dir)
        if os.path.isdir(local_dir):
            for root, dir, files in os.walk(local_dir):
                for filename in files:
                    fullpath = os.path.join(root, filename)
                    object_key = object_path + (fullpath[len(local_dir):].replace('\\', '/'))
                    if not any(item and len(item) > 0 and item in fullpath for item in excludes):
                        self.client.upload_file(fullpath, bucket_name, object_key)
                        logging.info("file: %s upload completed!", object_key)
        logging.info("dir: %s upload completed!", local_dir)

    def download_file(self, remote_file, local_file):
        bucket_name = extract_bucket(remote_file)
        object_key = extract_object_key(remote_file)
        try:
            self.client.download_file(bucket_name, object_key, local_file)
            logging.info("%s download completed!", local_file)
            return True
        except FileNotFoundError:
            logging.error("%s S3 object not found", object_key)
            return False
        except NoCredentialsError:
            logging.error("AWS no Credential")
            return False

    def has_object(self, bucket, prefix='', delimiter='/'):
        kwargs = {'Bucket': bucket, 'Prefix': prefix, 'Delimiter': delimiter, 'MaxKeys': 2}
        resp = self.client.list_objects_v2(**kwargs)
        return resp.get("KeyCount") > 0

    def get_matching_s3_prefix(self, bucket, prefix='', suffix='', delimiter='/'):
        """
        Generate the prefix in an S3 bucket.

        :param bucket: Name of the S3 bucket.
        :param prefix: Only fetch keys that start with this prefix (optional).
        :param suffix: Only fetch keys that end with this suffix (optional).
        :param delimiter: A delimiter is a character you use to group keys (optional).
        """
        kwargs = {'Bucket': bucket, 'Prefix': prefix, 'Delimiter': delimiter, 'MaxKeys': 10000}
        while True:
            resp = self.client.list_objects_v2(**kwargs)
            if resp.get("CommonPrefixes"):
                for dir in resp.get("CommonPrefixes"):
                    key = dir.get("Prefix")
                    if key.endswith(suffix):
                        yield key
            try:
                kwargs['ContinuationToken'] = resp['NextContinuationToken']
            except KeyError:
                break

    def scan_deep_s3_prefix(self, bucket, prefix='', layer=1, delimiter='/'):
        """
        Generate the prefix in an S3 bucket.

        :param bucket: Name of the S3 bucket.
        :param prefix: Only fetch keys that start with this prefix (optional).
        :param layer: Directory recursion depth (optional).
        :param delimiter: A delimiter is a character you use to group keys (optional).
        """
        if layer <= 0:
            return
        # 'MaxKeys': 1000
        # 'MaxKeys': 500 * Prefix.count(Delimiter)
        kwargs = {'Bucket': bucket, 'Prefix': prefix, 'Delimiter': delimiter}
        resp = self.client.list_objects_v2(**kwargs)
        # print(resp)
        while True:
            if resp.get("CommonPrefixes"):
                for dir in resp.get("CommonPrefixes"):
                    next_prefix = dir.get("Prefix")
                    if layer > 1:
                        yield from scan_deep_s3_keys(bucket, next_prefix, layer-1, delimiter)
                    else:
                        yield next_prefix
                #     # print(dir.get())
                try:
                    kwargs['ContinuationToken'] = resp['NextContinuationToken']
                except KeyError:
                    break
            else:
                break

    def get_matching_s3_keys(self, bucket, prefix='', suffix=''):
        """
        Generate the keys in an S3 bucket.

        :param bucket: Name of the S3 bucket.
        :param prefix: Only fetch keys that start with this prefix (optional).
        :param suffix: Only fetch keys that end with this suffix (optional).
        """

        # If the prefix is a single string (not a tuple of strings), we can
        # do the filtering directly in the S3 API.
        kwargs = {'Bucket': bucket, 'Prefix': prefix}
        while True:

            # The S3 API response is a large blob of metadata.
            # 'Contents' contains information about the listed objects.
            resp = self.client.list_objects_v2(**kwargs)
            for obj in resp['Contents']:
                key = obj['Key']
                if key.startswith(prefix) and key.endswith(suffix):
                    yield key

            # The S3 API is paginated, returning up to 1000 keys at a time.
            # Pass the continuation token into the next response, until we
            # reach the final page (when this field is missing).
            try:
                kwargs['ContinuationToken'] = resp['NextContinuationToken']
            except KeyError:
                break



# keys = get_matching_s3_keys('ds-prod-rawdata', 'data/', 4)
#
# # paths = list(itertools.chain.from_iterable(keys))
# paths = list(keys)
# # print(sum(1 for _ in keys))
# print(len(paths))
# print(paths)



# paginator = s3.get_paginator("list_objects_v2")
#
# for page in paginator.paginate(Bucket="ds-dev-devops"):
#     print(page["Contents"])


import tarfile
import os


def make_tar(tar_name, src_dirs, mode='w:gz', excludes=[]):
    """
    :param tar_name:
    :param src_dirs: []
    :param mode:
    :param excludes: [] filename
    :return:
    """
    tar = tarfile.open(tar_name, mode)
    for src_dir in src_dirs:
        if os.path.isdir(src_dir):
            for root, dir, files in os.walk(src_dir):
                for filename in files:
                    if not any(item in filename for item in excludes):
                        fullpath = os.path.join(root, filename)
                        tar.add(fullpath, arcname=filename)
        elif os.path.isfile(src_dir):
            filename = os.path.basename(src_dir)
            if not any(filename in item for item in excludes):
                tar.add(src_dir, arcname=filename)
    tar.close()


def un_tar(tar_path, dist_dir, mode='r:gz'):
    """
    :param tar_path:
    :param dist_dir:
    :param mode:
    :return:
    """
    with tarfile.open(tar_path, mode) as tar:
        tar.extractall(path=dist_dir)


# make_tar('D:/mnt/data/private/model/20230619/model.tar.gz', ['D:/mnt/data/private/model/20230619'])
# un_tar('D:/mnt/data/private/model/20230607/model.tar.gz', 'D:/mnt/data/private/temp/20230607')




from setuptools import setup, find_packages
import os


setup(
    name="bestv_common_util",
    version="0.3.8",
    author="liu.qingchen",
    author_email="liu.qingchen@bestv.com.cn",
    description="Bestv Common Util",
    long_description_content_type="text/markdown",
    long_description=open('README.md', encoding="UTF8").read(),
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=['requests', 'pandas'],
    keywords=['bestv_common_util', 'nacos', 'kms'],
    # data_files=[('cut_video', ['cut_video/clip_to_erase.json'])],
    entry_points={
        'console_scripts': [

        ]
    },
    license="MIT",
    url="",
    scripts=[],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows"
    ]
)


#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
脚本名称：properties_handler.py
脚本功能：解析.properties配置文件
编写人：  pangtaishi
编写日期：2021-02-28
'''

import os
import sys
import logging


def __get_dict(str_name, dict_name, value):
    if str_name.find('.') > 0:
        k = str_name.split('.')[0]
        dict_name.setdefault(k, {})
        return __get_dict(str_name[len(k) + 1:], dict_name[k], value)
    else:
        dict_name[str_name] = value
        return


def load_dict(filename):
    props_file = open(filename, 'Ur')
    props = parse_properties(props_file.readlines())
    props_file.close()
    return props


def parse_dict(lines):
    props = {}
    for line in lines:
        line = line.strip().replace('\n', '')
        if line.find("#") != -1:
            line = line[0:line.find('#')]
        if line.find('=') > 0:
            strs = line.split('=')
            strs[1] = line[len(strs[0]) + 1:]
            __get_dict(strs[0].strip().strip("'").strip(), props, strs[1].strip().strip("'").strip())
    return props


def load_properties(filename):
    props_file = open(filename, 'Ur')
    props = parse_properties(props_file.readlines())
    props_file.close()
    return props


def parse_properties(lines):
    props = {}
    for line in lines:
        line = line.strip().replace('\n', '')
        if line.find("#") != -1:
            line = line[0:line.find('#')]
        if line.find('=') > 0:
            strs = line.split('=')
            strs[1] = line[len(strs[0]) + 1:]
            props[strs[0].strip().strip("'").strip()] = strs[1].strip().strip("'").strip()
    return props


def properties_to_dict(props):
    dict = {}
    if props and props.items():
        for key, value in props.items():
            __get_dict(key.strip().strip("'").strip(), dict, value.strip().strip("'").strip())
    return dict


def flatten_dict(d, parent_key='', sep='.'):
    items = []
    for k, v in d.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, dict):
            items.extend(flatten_dict(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


def yaml_to_properties(yaml_str):
    import yaml
    data = yaml.safe_load(yaml_str)
    flat_data = flatten_dict(data)
    properties = "\n".join([f"{k}={v}" for k, v in flat_data.items()])
    return properties


def get_properties_by_prefix(props: dict, prefix: str):
    if props and prefix:
        prefix_len = len(prefix)
        dict = {}
        if props and props.items():
            for key, value in props.items():
                if key.startswith(prefix):
                    dict[key[prefix_len:]] = value
        return dict
    return props